@@ -2,6 +2,7 @@ import os
 import sys
 from urllib.request import urlopen
 
+
 _conf_url = \
         "https://raw.githubusercontent.com/inducer/sphinxconfig/main/sphinxconfig.py"
 with urlopen(_conf_url) as _inf:
@@ -22,6 +23,6 @@ version = ver_dic["VERSION"]
 release = version
 
 intersphinx_mapping = {
-    "https://docs.python.org/3": None,
-    "urwid": ("https://urwid.org/", None),
-    }
+        "python": ("https://docs.python.org/3", None),
+        "urwid": ("https://urwid.org/", None),
+        }
