@@ -1,8 +1,9 @@
 def f():
-    fail
+    fail  # noqa: B018, F821
+
 
 try:
     f()
-except:
+except Exception:
     from pudb import post_mortem
     post_mortem()
