@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 __copyright__ = """
 Copyright (C) 2009-2017 Andreas Kloeckner
 Copyright (C) 2014-2017 Aaron Meurer
@@ -27,8 +25,9 @@ THE SOFTWARE.
 
 
 import logging
+import sys
 from datetime import datetime
-from pudb.py3compat import PY3, text_type
+
 
 logfile = [None]
 
@@ -52,7 +51,7 @@ class TerminalOrStreamHandler(logging.StreamHandler):
     stderr, depending on whether the debugger is active.
     """
     def emit(self, record):
-        from pudb import _have_debugger, _get_debugger
+        from pudb import _get_debugger, _have_debugger
         logfile = getlogfile()
 
         self.acquire()
@@ -66,7 +65,7 @@ class TerminalOrStreamHandler(logging.StreamHandler):
                 message = self.format(record)
                 dbg.ui.add_cmdline_content(message, "command line error")
             else:
-                super(TerminalOrStreamHandler, self).emit(record)
+                super().emit(record)
         finally:
             self.release()
 
@@ -97,26 +96,22 @@ ui_log, settings_log = _init_loggers()
 # {{{ breakpoint validity
 
 def generate_executable_lines_for_code(code):
-    lineno = code.co_firstlineno
-    yield lineno
-    if PY3:
+    if sys.version_info >= (3, 10):
+        for _start, _end, lineno in code.co_lines():
+            if lineno is not None:
+                yield lineno
+    else:
+        lineno = code.co_firstlineno
+        yield lineno
         # See https://github.com/python/cpython/blob/master/Objects/lnotab_notes.txt
-        import sys
-        use_36_line_incr = sys.version_info >= (3, 6)
 
         for line_incr in code.co_lnotab[1::2]:
-            # This showed up between the v3.5 and v3.6 cycles:
-            # https://github.com/python/cpython/blob/v3.5.0/Objects/lnotab_notes.txt
+            # NB: This code is specific to Python 3.6 and higher
             # https://github.com/python/cpython/blob/v3.6.0/Objects/lnotab_notes.txt
-            # Gate the check on 3.6-or-newer.
-            if line_incr >= 0x80 and use_36_line_incr:
+            if line_incr >= 0x80:
                 line_incr -= 0x100
             lineno += line_incr
             yield lineno
-    else:
-        for c in code.co_lnotab[1::2]:
-            lineno += ord(c)
-            yield lineno
 
 
 def get_executable_lines_for_codes_recursive(codes):
@@ -177,7 +172,7 @@ def lookup_module(filename):
     f = os.path.join(sys.path[0], filename)
     if os.path.exists(f):  # and self.canonic(f) == self.mainpyfile:
         return f
-    root, ext = os.path.splitext(filename)
+    _root, ext = os.path.splitext(filename)
     if ext == "":
         filename = filename + ".py"
     if os.path.isabs(filename):
@@ -197,14 +192,16 @@ def lookup_module(filename):
 # the main idea stolen from Python 3.1's tokenize.py, by Ka-Ping Yee
 
 import re
+
+
 cookie_re = re.compile(br"^\s*#.*coding[:=]\s*([-\w.]+)")
-from codecs import lookup, BOM_UTF8
+from codecs import BOM_UTF8, lookup
 
 
 def detect_encoding(line_iter):
     """
     The detect_encoding() function is used to detect the encoding that should
-    be used to decode a Python source file. It requires one argment, line_iter,
+    be used to decode a Python source file. It requires one argument, line_iter,
     an iterator on the lines to be read.
 
     It will read a maximum of two lines, and return the encoding used
@@ -228,10 +225,7 @@ def detect_encoding(line_iter):
 
     def find_cookie(line):
         try:
-            if PY3:
-                line_string = line
-            else:
-                line_string = line.decode("ascii")
+            line_string = line
         except UnicodeDecodeError:
             return None
 
@@ -241,9 +235,9 @@ def detect_encoding(line_iter):
         encoding = matches[0].decode()
         try:
             codec = lookup(encoding)
-        except LookupError:
+        except LookupError as err:
             # This behaviour mimics the Python interpreter
-            raise SyntaxError("unknown encoding: " + encoding)
+            raise SyntaxError("unknown encoding: " + encoding) from err
 
         if bom_found and codec.name != "utf-8":
             # This behaviour mimics the Python interpreter
@@ -251,7 +245,7 @@ def detect_encoding(line_iter):
         return encoding
 
     first = read_or_stop()
-    if isinstance(first, text_type):
+    if isinstance(first, str):
         return None, [first]
 
     if first.startswith(BOM_UTF8):
@@ -289,38 +283,4 @@ def decode_lines(lines):
 
 # }}}
 
-
-# {{{ traceback formatting
-
-class StringExceptionValueWrapper:
-    def __init__(self, string_val):
-        self.string_val = string_val
-
-    def __str__(self):
-        return self.string_val
-
-    __context__ = None
-    __cause__ = None
-
-
-def format_exception(exc_tuple):
-    # Work around http://bugs.python.org/issue17413
-    # See also https://github.com/inducer/pudb/issues/61
-
-    from traceback import format_exception
-    if PY3:
-        exc_type, exc_value, exc_tb = exc_tuple
-
-        if isinstance(exc_value, str):
-            exc_value = StringExceptionValueWrapper(exc_value)
-            exc_tuple = exc_type, exc_value, exc_tb
-
-        return format_exception(
-                *exc_tuple,
-                **dict(chain=hasattr(exc_value, "__context__")))
-    else:
-        return format_exception(*exc_tuple)
-
-# }}}
-
 # vim: foldmethod=marker
