@@ -1,6 +1,10 @@
-# -*- coding: utf-8 -*-
+"""
+.. autoclass:: RemoteDebugger
 
-from __future__ import absolute_import, print_function, unicode_literals
+.. autofunction:: set_trace
+.. autofunction:: debugger
+.. autofunction:: debug_remote_on_single_rank
+"""
 
 __copyright__ = """
 Copyright (C) 2009-2017 Andreas Kloeckner
@@ -32,27 +36,30 @@ THE SOFTWARE.
 # mostly stolen from celery.contrib.rdb
 
 
+import atexit
 import errno
 import os
 import socket
 import sys
-import fcntl
-import termios
-import struct
+from typing import Any, Callable
 
 from pudb.debugger import Debugger
 
-__all__ = ["PUDB_RDB_HOST", "PUDB_RDB_PORT", "default_port", "debugger", "set_trace"]
+
+__all__ = ["PUDB_RDB_HOST", "PUDB_RDB_PORT", "PUDB_RDB_REVERSE",
+           "default_port", "debugger", "set_trace",
+           "debug_remote_on_single_rank"]
 
 default_port = 6899
 
 PUDB_RDB_HOST = os.environ.get("PUDB_RDB_HOST") or "127.0.0.1"
 PUDB_RDB_PORT = int(os.environ.get("PUDB_RDB_PORT") or default_port)
+PUDB_RDB_REVERSE = bool(os.environ.get("PUDB_RDB_REVERSE"))
 
 #: Holds the currently active debugger.
 _current = [None]
 
-_frame = getattr(sys, "_getframe")
+_frame = sys._getframe
 
 NO_AVAILABLE_PORT = """\
 {self.ident}: Couldn't find an available port.
@@ -61,7 +68,8 @@ Please specify one using the PUDB_RDB_PORT environment variable.
 """
 
 BANNER = """\
-{self.ident}: Please telnet into {self.host} {self.port}.
+{self.ident}: Please start a telnet session using a command like:
+telnet {self.host} {self.port}
 {self.ident}: Waiting for client...
 """
 
@@ -79,7 +87,35 @@ Please specify one using the PUDB_RDB_PORT environment variable.
 """
 
 
+class TelnetCharacters:
+    """Collection of characters from the telnet protocol RFC 854
+
+    This format for the telnet characters was adapted from the telnetlib module
+    which was removed from the C Python standard library in version 3.13. Only
+    the characters needed by pudb have been copied here. Additional characters
+    can be found by looking in the telnetlib code in Python 3.12 or in the
+    telnet RFC.
+
+    .. note::
+
+        This class is not intended to be instantiated.
+    """
+    # Telnet protocol characters
+    IAC = b"\xff"  # "Interpret As Command"
+    DO = b"\xfd"
+    WILL = b"\xfb"
+
+    # Telnet protocol options codes
+    # These ones all come from arpa/telnet.h
+    ECHO = b"\x01"  # echo
+    SGA = b"\x03"  # suppress go ahead
+
+
 class RemoteDebugger(Debugger):
+    """
+    .. automethod:: __init__
+    """
+
     me = "pudb"
     _prev_outs = None
     _sock = None
@@ -91,11 +127,32 @@ class RemoteDebugger(Debugger):
         port_search_limit=100,
         out=sys.stdout,
         term_size=None,
-        reverse=False,
+        reverse=PUDB_RDB_REVERSE,
     ):
-        self.active = True
+        """
+        :arg term_size: A two-tuple ``(columns, rows)``, or *None*. If *None*,
+            try to determine the terminal size automatically.
+
+            Currently, this uses a heuristic: It uses the terminal size of the
+            debuggee as that for the debugger. The idea is that you might be
+            running both in two tabs of the same terminal window, hence using
+            terminals of the same size.
+        """
         self.out = out
 
+        if term_size is None:
+            term_size = os.environ.get("PUDB_TERM_SIZE")
+            if term_size is not None:
+                term_size = tuple(map(int, term_size.split("x")))
+                if len(term_size) != 2:
+                    raise ValueError("PUDB_TERM_SIZE should have two dimensions")
+            else:
+                try:
+                    s = os.get_terminal_size()
+                    term_size = (s.columns, s.lines)
+                except Exception:
+                    term_size = (80, 24)
+
         self._prev_handles = sys.stdin, sys.stdout
         self._client, (address, port) = self.get_client(
             host=host, port=port, search_limit=port_search_limit, reverse=reverse
@@ -108,24 +165,14 @@ class RemoteDebugger(Debugger):
         raw_sock_file = self._client.makefile("rwb", 0)
         import codecs
 
-        if sys.version_info[0] < 3:
-            sock_file = codecs.StreamRecoder(
-                raw_sock_file,
-                codecs.getencoder("utf-8"),
-                codecs.getdecoder("utf-8"),
-                codecs.getreader("utf-8"),
-                codecs.getwriter("utf-8"),
-            )
-        else:
-            sock_file = codecs.StreamReaderWriter(
-                raw_sock_file, codecs.getreader("utf-8"), codecs.getwriter("utf-8")
-            )
+        sock_file = codecs.StreamReaderWriter(
+            raw_sock_file, codecs.getreader("utf-8"), codecs.getwriter("utf-8"))
 
         self._handle = sys.stdin = sys.stdout = sock_file
 
         # nc negotiation doesn't support telnet options
         if not reverse:
-            import telnetlib as tn
+            tn = TelnetCharacters
 
             raw_sock_file.write(tn.IAC + tn.WILL + tn.SGA)
             resp = raw_sock_file.read(3)
@@ -143,13 +190,13 @@ class RemoteDebugger(Debugger):
         if reverse:
             self.host, self.port = host, port
             client, address = self.get_reverse_socket_client(host, port)
-            self.ident = "{0}:{1}".format(self.me, self.port)
+            self.ident = f"{self.me}:{self.port}"
         else:
             self._sock, conn_info = self.get_socket_client(
                 host, port, search_limit=search_limit,
             )
             self.host, self.port = conn_info
-            self.ident = "{0}:{1}".format(self.me, self.port)
+            self.ident = f"{self.me}:{self.port}"
             self.say(BANNER.format(self=self))
             client, address = self._sock.accept()
         client.setblocking(1)
@@ -160,9 +207,9 @@ class RemoteDebugger(Debugger):
         try:
             _sock.connect((host, port))
             _sock.setblocking(1)
-        except socket.error as exc:
+        except OSError as exc:
             if exc.errno == errno.ECONNREFUSED:
-                raise ValueError(CONN_REFUSED.format(self=self))
+                raise ValueError(CONN_REFUSED.format(self=self)) from exc
             raise exc
         return _sock, _sock.getpeername()
 
@@ -180,7 +227,7 @@ class RemoteDebugger(Debugger):
             this_port = port + i
             try:
                 _sock.bind((host, this_port))
-            except socket.error as exc:
+            except OSError as exc:
                 if exc.errno in [errno.EADDRINUSE, errno.EINVAL]:
                     continue
                 raise
@@ -192,56 +239,66 @@ class RemoteDebugger(Debugger):
     def say(self, m):
         print(m, file=self.out)
 
-    def _close_session(self):
+    def close_remote_session(self):
         self.stdin, self.stdout = sys.stdin, sys.stdout = self._prev_handles
         self._handle.close()
         self._client.close()
-        self._sock.close()
-        self.active = False
+        if self._sock:
+            self._sock.close()
         self.say(SESSION_ENDED.format(self=self))
 
-    def do_continue(self, arg):
-        self._close_session()
-        self.set_continue()
-        return 1
 
-    do_c = do_cont = do_continue
-
-    def do_quit(self, arg):
-        self._close_session()
-        self.set_quit()
-        return 1
-
-    def set_quit(self):
-        # this raises a BdbQuit exception that we are unable to catch.
-        sys.settrace(None)
-
-
-def debugger(term_size=None, host=PUDB_RDB_HOST, port=PUDB_RDB_PORT, reverse=False):
+def debugger(
+    term_size=None,
+    host=PUDB_RDB_HOST,
+    port=PUDB_RDB_PORT,
+    reverse=PUDB_RDB_REVERSE
+):
     """Return the current debugger instance (if any),
     or creates a new one."""
     rdb = _current[0]
-    if rdb is None or not rdb.active:
+    if rdb is None:
         rdb = _current[0] = RemoteDebugger(
             host=host, port=port, term_size=term_size, reverse=reverse
         )
+        atexit.register(lambda e: e.close_remote_session(), rdb)
     return rdb
 
 
 def set_trace(
-    frame=None, term_size=None, host=PUDB_RDB_HOST, port=PUDB_RDB_PORT, reverse=False
+    frame=None,
+    term_size=None,
+    host=PUDB_RDB_HOST,
+    port=PUDB_RDB_PORT,
+    reverse=PUDB_RDB_REVERSE
 ):
     """Set breakpoint at current location, or a specified frame"""
     if frame is None:
         frame = _frame().f_back
-    if term_size is None:
-        try:
-            # Getting terminal size
-            s = struct.unpack("hh", fcntl.ioctl(1, termios.TIOCGWINSZ, "1234"))
-            term_size = (s[1], s[0])
-        except Exception:
-            term_size = (80, 24)
 
     return debugger(
         term_size=term_size, host=host, port=port, reverse=reverse
     ).set_trace(frame)
+
+
+def debug_remote_on_single_rank(comm: Any, rank: int, func: Callable,
+                                *args: Any, **kwargs: Any) -> None:
+    """Run a remote debugger on a single rank of an ``mpi4py`` application.
+    *func* will be called on rank *rank* running in a :class:`RemoteDebugger`,
+    and will be called normally on all other ranks.
+
+    :param comm: an ``mpi4py`` ``Comm`` object.
+    :param rank: the rank to debug. All other ranks will spin until this rank exits.
+    :param func: the callable to debug.
+    :param args: the arguments passed to ``func``.
+    :param kwargs: the kwargs passed to ``func``.
+    """
+    if comm.rank == rank:
+        debugger().runcall(func, *args, **kwargs)
+    else:
+        try:
+            func(*args, **kwargs)
+        finally:
+            from time import sleep
+            while True:
+                sleep(1)
